* ---------------------------------------------------------------- smc_snip
      Program SMC_Snip

* Snip out a section of an SMC and write the results as an SMC file.

* Input/output controlled by information in a control file.  Here is an example:

*Control file for program SMC_SNIP
*f_smc_in tskip tlength f_smc_out ("STOP" in any column to quit)
* file1_in.smc 0.0  80.0 file1_out.smc
* file2_in.smc 0.0  80.0 file2_out.smc
* file3_in.smc 0.0  80.0 file3_out.smc
*StoP
* file4_in.smc 0.0  80.0 file4_out.smc

* Dates: 05/13/99 - Written by D. Boore
*        11/23/99 - Renamed (old version was RefrmSMC), and obtain parameters 
*                   interactively from the user.
*        02/08/00 - Renamed from snip_smc, and include latest version of 
*                   smcread, smcwrite
*        11/09/00 - Increase size of array y from 24000 to 100000; use
*                   a control file to bring in information.
*                   Also do a better job of adding comments, using existing
*                   itype if it exists
*        12/04/00 - added ibgn, iend to rcf, rci, rcc calling arguments
*        12/07/00 - Add an initial line to control file and
*                   use allocatable array
*        05/07/02 - Suggest control file name
*        10/22/02 - Added stuff to comment headers
*        07/17/03 - Add sps to smc_npts parameter list
*        12/17/03 - Modifications to be able to use longer file names 
*                   (compiled using LF95)
*        06/22/04 - Check for noninteger char_head(1) and set itype to 0
*        07/11/07 - Allow for longer file names
*        10/16/07 - Transparent change to user needed because smcread can
*                   read two input formats
!        09/13/12 - Inquire for existence of input file; various minor 
!                   modernizations to code (labeled loops, remove statement numbers, etc.)
!        06/28/19 - Increase lengths of in and out files, the buf file. Also, had to limit the
!                   comment with the out file name to 80 characters, otherwise I had a bug


      character f_smc_in*300, f_smc_out*300
      character char_head(11)*80, comments(500)*80
      integer int_head(48), ios
      real real_head(50)
      real y(:)
      allocatable :: y
      character buf*500, buf4*4
      character precision*8

      character date_c*8, time_c*11

      character f_ctl*40
      logical f_ctl_exist, f_in_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (Enter = smc_snip.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl == ' ') f_ctl = 'smc_snip.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skip(nu_ctl,2)
      

      loop over input files: DO


        buf = ' '
        read(nu_ctl,'(a)', iostat=ios) buf
        if (ios /= 0) then  ! reached end of file list
          EXIT loop over input files
        end if
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(buf4(1:4) == 'STOP') then
          EXIT loop over input files
        end if
  
        write(*,'(3a,1x,i3)') 
     :  ' buf = ', buf(1:nc_buf), ' and nc_buf = ', nc_buf

        f_smc_in = ' '
        call RCC (1,buf,nc_buf,f_smc_in,nc_f_in,ibgn,iend)
  
        f_in_exist = .false.
        inquire(file=f_smc_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ** Input file '//f_smc_in(1:nc_f_in)//
     :    ' does not exist, moving to next file **'
          CYCLE loop over input files
        end if

        print *, ' Processing file: '//f_smc_in(1:nc_f_in)
        call smc_npts(f_smc_in(1:nc_f_in), npts_max, sps)
  
        allocate (y(npts_max))
        
        call RCF (2,buf,nc_buf,tskip,ibgn,iend,istat)
  
        call RCF (3,buf,nc_buf,tlength,ibgn,iend,istat)
  
        f_smc_out = ' '
        call RCC (4,buf,nc_buf,f_smc_out,nc_f_out,ibgn,iend)

        call SMCRead(f_smc_in(1:nc_f_in), tskip, tlength,
     :             y, char_head, int_head, real_head, comments)
     
        ncomments = int_head(16)
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)')
     : '| Using program SMC_Snip'
        ncomments = ncomments + 1
        comments(ncomments) = 
     : '| file '//f_smc_out(1:nc_f_out)//' made'
        iflag = 0 ! not used by current PC version of datetime; Chris
                  ! Stephens would modify datetime.for to include Unix routines
        call datetime(date_c, time_c, iflag)
      
        ncomments = ncomments + 1
        comments(ncomments) = 
     : '| on '//date_c//' at time '//time_c
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,es12.5,a)')
     : '| by snipping ', tlength, ' sec'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a80)')
     : '| out of file '//f_smc_in(1:nc_f_in)
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)')
     : '| starting at ', tskip, ' sec'
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        int_head(16) = ncomments

        if (ichar(char_head(1)(1:1)) < 48 .or.
     :    ichar(char_head(1)(1:1)) > 57 ) then  ! not an integer
          write(*,'(a)') ' char_head(1)(1:1), not integer; = '//
     :                   char_head(1)(1:1)
          itype = 0
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)')
     :    '| char_head(1) was noninteger; setting '//
     :      'itype to 0 (unknown)'
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          int_head(16) = ncomments
        else
          read(char_head(1)(1:1), '(i1)') itype
          write(*,'(a)') ' char_head(1)(1:1), integer; = '//
     :                   char_head(1)(1:1)
          if(itype >= 0 .and. itype <= 5) then
            itype = -1
          else
            itype =  0
            ncomments = ncomments + 1
            write(comments(ncomments),'(a)')
     :      '| and setting itype to 0 (unknown)'
            ncomments = ncomments + 1
            comments(ncomments) = '|'
            int_head(16) = ncomments
          end if
        end if


        precision = ' '
        if (int_head(47) == 8) then
          precision = 'HIGH'
        else
          precision = 'STANDARD'
        end if
      
        call SMCWrite(f_smc_out(1:nc_f_out), precision,
     :             y, char_head, itype, int_head, real_head, comments)

        deallocate (y)
      
      END DO loop over input files

      close(nu_ctl)
  
      end
* ---------------------------------------------------------------- smc_snip
      
      include '\forprogs\get_lun.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\SMCRead.for'
      include '\forprogs\SMCWrite.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\skip.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\datetime.for'
      include '\forprogs\distaz.for'



