
       Program Add_vs30_2SMC

*Reads in a table containing a station code and Vs30, and then adds the Vs30
*to an smc file, storing the value in real_head(40).
*The Vs30 table has entries following an initial header; the
*station code and Vs30 values can be in arbitrary columns.
*To be successful, the station code in the smc file is obtained
*from the first ten characters of the text_head(3)

*!Control file for program Add_vs30_2SMC
*!Name of summary file:
* Add_vs30_2SMC.sum
*!Name of file with v30 data
* vs30_ctlee_18april08.txt
*! column numbers containing staname, vs30
*  1 5
*!smc filename
*P1_T130N.ASC.smc8
*P1_K042N.ASC.smc8
*P1_K042V.ASC.smc8
*stop
 
*  Dates:  04/22/08 - Written by D. Boore, based on taiwan_one_component_2smc.for
*          08/01/08 - Program Taiwan_add_vs30_3smc, renamed with some explanatory info.
 
      character precision*20
      character cmnts2skip(200)*79, read_fmt*50, f_vs30*120,
     :          buf10*10, buf*80, buf4*4
       
      real y(:)
      allocatable :: y

      character date_c*8, time_c*11

      
      real  rhead(50), sps, orient_h, orient_v, eqmag, 
     :      eqlat, eqlong, rnull

      integer ihead(48), iyr, imon, idom, idoy, ihr, isec,
     :        ncomments, inull

      integer idata(100000) 
      real rdata(100000)

      character f_ctl*100, f_sum*100, f_smc*100, f_in*100 

      character chead(11)*80, comp_c*1, staname*10,  
     :          comments(50)*80, ins_type*5, 
     :          type_c*40, temp_c*10
     
      character staname_vs30(1000)*10
      integer nc_staname_vs30(1000)
      real vs30(1000)

      logical file_exist

      inull = -32768
      rnull = +1.7e+38

* Open control file:

      file_exist = .false.
      do while (.not. file_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file'//
     :    ' (cr = add_vs30_2SMC.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'add_vs30_2SMC.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=file_exist)
        if (.not. file_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum, file=f_sum(1:nc_f_sum), status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      f_vs30 = ' '
      read(nu_ctl,'(a)') f_vs30
      call trim_c(f_vs30,nc_f_vs30)
      call get_lun(nu_vs30)
      open(unit=nu_vs30, file=f_vs30(1:nc_f_vs30), status='unknown')
      write(nu_sum,'(a)') ' Vs30 file: '//f_vs30(1:nc_f_vs30)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      read(nu_ctl,*) icol_staname, icol_vs30
      write(nu_sum,*) icol_staname, icol_vs30

* Read values of vs30 file into arrays
      call skip(nu_vs30, 1) 
      
      nvs30 = 0
      
5000  continue
      buf = ' '
      read(nu_vs30,'(a)', end=5555) buf
      call trim_c(buf, nc_buf)
      
      nvs30 = nvs30 + 1
      staname_vs30(nvs30) = ' '
      call rcc(icol_staname,buf,nc_buf,staname_vs30(nvs30),nc_st,
     :         ibgn,iend)
      call upstr(staname_vs30(nvs30))
      call trim_c(staname_vs30(nvs30), nc_staname_vs30(nvs30))

      call rcf(icol_vs30,buf,nc_buf,vs30(nvs30),ibgn,iend, ISTAT)
      
 
      go to 5000
5555  continue

      write(nu_sum,*)
      write(nu_sum,'(a)') ' i, staname, vs30'
      
      do i = 1, nvs30
        write(nu_sum,*) i, staname_vs30(i)(1:nc_staname_vs30(i)), 
     :                     vs30(i)
      end do

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  

1000  continue

        buf = ' '
        read(nu_ctl,'(a)') buf
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call trim_c(buf4,nc_buf4)
        call upstr(buf4)
      
        if (buf4(1:4) .eq. 'STOP') then
          close(nu_ctl)
          close(nu_sum)
          stop
        end if
      
        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in, nc_f_in)

        call SMC_Npts(f_in(1:nc_f_in),npts,sps)
        allocate( y(npts))
        tskip = 0.0
        tlength = -1.0
        call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :             y, chead, ihead, rhead, comments)

        precision = ' '
        if (ihead(47) .eq. 8) then
          precision = 'HIGH'
        else
          precision = 'STANDARD'
        end if
      
        staname = ' '
        staname = chead(3)
        call upstr(staname)
        call trim_c(staname, nc_staname)
        
* Find staname and store vs30 in rhead(40)

        do i = 1, nvs30
          if (staname(1:nc_staname) .eq. 
     :            staname_vs30(i)(1:nc_staname_vs30(i))) then
     
            rhead(40) = vs30(i)
* Set up comments:
        ncomments = ihead(16)
        ncomments = ncomments + 1
        comments(ncomments) = '| Added Vs30 to real_head(40) from '//
     :                           f_vs30(1:nc_f_vs30)
        iflag = 0
        call datetime(date_c, time_c, iflag)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :  '|   file written on '//date_c//' at time '//time_c
        ihead(16) = ncomments
             
            go to 6666
          end if
        end do
6666    continue

* Write smc file

 
      itype = 1
      call smcwrite(f_in(1:nc_f_in), precision,
     :              y, chead, itype, 
     :              ihead, rhead, comments)
     
      deallocate(y)
     
* That should be it; loop back for another component

      go to 1000

 
      end

      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\doy.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\distaz.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\datetime.for'
 

