      Program SMC_DIG

* Read in a smc file, apply analog-to-digital conversion, and write as a new 
* smc file.

* Input from a control file. Here is an example:
*! Control file for program SMC_Dig
*!Place in which to overwrite existing character with stem that follows
*!after period (-1 means just to the left; 
*!0 will insert to the left of the period unless the stem is 7 or 8 characters,
*!in which case it will overwrite the 7th & 8th characters):
* -3 
*!String to write into input name:
* d1
*!
*! y_full_scale, nbits, exceedance (0) or round (.ne.0).  
*! (dy = 2*y_full_scale/2.0**float(nbits))
*  1962.0 16 0
*! Sigma (as a multiple of the quanta dy) to use in dithering 
*! (0 = no dithering), iseed
* 0.67  345 
*! Names of input files
*smc1.smc
*smc2.smc
*Stop


* Dates: 12/19/01 - Revised by D. Boore, written a week ago
*        12/20/01 - Added option to use either roundoff or exceedance
*        07/27/02 - Read in dy rather than nbits, y_full_scale, and
*                   include options for constructing output file name.
*        08/12/02 - Renamed from smc_a2d, because that name might
*                   be interpreted as "acceleration to displacement", since
*                   I recently wrote smc_v2a to convert velocity 
*                   to acceleration.
*        09/04/02 - I changed the relation between dy, y_full_scale, nbits
*                   to reflect +-y_full_scale (the equation is the same,
*                   but I move a factor of 2 to the numerator).
*        10/03/02 - Extensive revision to change the digitization to more
*                   accurately portray the ADC used by the A-900 (before
*                   I used aint, which truncates toward 0.0; what I want is
*                   a "floor" function instead).
*        10/18/02 - Added dithering
*        10/22/02 - Add stuff to comment headers
*        07/17/03 - Add sps to smc_npts parameter list
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats

      character precision*8
      
 
      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      integer int_head(48)
      real real_head(50)
      real y_in(:), y_out(:)
      allocatable :: y_in, y_out
      character buf*80, buf4*4, f_in*40, f_out*40, astring*4

      character fl_rd_c*2, type_of_a2d*10

      character date_c*8, time_c*11

      character f_ctl*40
      logical f_ctl_exist, round, f_in_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=smc_dig.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc_dig.ctl'
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(nu_ctl,f_ctl,status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl, *) n4name
      write(*,'(a,1x,i3)') ' n4name = ', n4name

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      astring = ' '
      read(nu_ctl,'(a)') astring
      call trim_c(astring, nc_astring)

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) y_full_scale, nbits, iround
      write(*,'(a)') ' y_full_scale, nbits, iround = '
      write(*,*) y_full_scale, nbits, iround
      dy = 2*y_full_scale/2.0**float(nbits)

      if (iround .eq. 0) then
        round = .false.
        fl_rd_c = 'fl'
        type_of_a2d = ' floor a2d'
      else
        round = .true.
        fl_rd_c = 'rd'
        type_of_a2d = ' round a2d'
      end if
      write(*,'(a)') 
     :   ' fl_rd_c, type_of_a2d = '//fl_rd_c//', '//type_of_a2d

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      read(nu_ctl,*) sigma_dither, iseed_dither_in
      write(*,'(a)') ' sigma_dither, iseed_dither = '
      write(*,*) sigma_dither, iseed_dither_in
      iseed_dither = -iabs(iseed_dither_in)

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)

      nfiles = 0

1000  continue

        buf = ' '
        read(nu_ctl,'(a)') buf
        call trim_c(buf, nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if (buf4 .eq. 'STOP') then
          close(nu_ctl)
          stop
        end if

        f_in = ' '
        f_in = buf(1:nc_buf)
        call trim_c(f_in,nc_f_in)
        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
            write(*,'(a)') ' ****** FILE '//f_in(1:nc_f_in)// 
     :            ' DOES NOT EXIST; SKIPPING!!! ******* '
            go to 1000
        end if

* Passed tests; proceed

        write(*,*)
        write(*,'(1x,a)') 
     :     'Processing file:', f_in(1:nc_f_in)

        nfiles = nfiles + 1

        call smc_npts(f_in(1:nc_f_in), npts, sps)

        write(*,'(a,1x,i8)') 
     :    ' For file '//f_in(1:nc_f_in)//', npts = ', npts

        allocate(y_in(npts), y_out(npts))

        tskip = 0.0
        tlength = -1.0
        call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :             y_in, char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
        ny = int_head(17)
        write(*,'(a,1x,i8)') ' For file '//f_in(1:nc_f_in)//
     :                     ', ny = ', ny

        if (sigma_dither .ne. 0.0) then
          do i = 1, ny
            y_in(i) = y_in(i) + sigma_dither*dy*gasdev(iseed_dither)
          end do
          indp = index(f_in,'.')
          f_out = ' '
          f_out = f_in(1:nc_f_in)
          f_out(indp+2:indp+3) = 'di'
          call trim_c(f_out,nc_f_out)
          ncomments = int_head(16)
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          ncomments = ncomments + 1
          comments(ncomments) = '|  Input file '//f_in(1:nc_f_in)
          ncomments = ncomments + 1
          comments(ncomments) = '|  dithered within program SMC_DIG '
          ncomments = ncomments + 1
          comments(ncomments) = 
     :     '|  with sigma_dither and iseed_dither = '
            write(comments(ncomments)(43:54),
     :      '(f6.3,1x,i5)') 
     :       sigma_dither, iseed_dither_in
          iflag = 0
          call datetime(date_c, time_c, iflag)
          ncomments = ncomments + 1
          comments(ncomments) = 
     :      '|   file '//f_out(1:nc_f_out)//
     :      ' written on '//date_c//' at time '//time_c
          ncomments = ncomments + 1
          comments(ncomments) = '|'
          int_head(16) = ncomments

          itype = -1  ! use info from header
          call SMCWrite(f_out(1:nc_f_out), precision,
     :                y_in, char_head, itype, 
     :                      int_head, real_head, comments)

        end if

        call digitize(y_in, y_out, ny, dy, round)

* DEBUG
*      do i = 1, ny
*        if (y_out(i) .ne. 0.0) then
*          write(*,'(a,1x,i5,1p,2(1x,e10.3))') 
*     :     ' i, y_in, y_out = ',
*     :       i, y_in(i), y_out(i)
*        end if
*      end do
* DEBUG

* Write the new smc file:

* Construct output filename:

        indp = index(f_in,'.')
        f_out = ' '
        f_out = f_in(1:nc_f_in)
        f_out(indp+n4name:indp+n4name+nc_astring-1) = 
     :        astring(1:nc_astring)
        
        call trim_c(f_out, nc_f_out)

        write(*,'(1x,a)') 
     :     'Writing output to file:', f_out(1:nc_f_out)

        ncomments = int_head(16)
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        ncomments = ncomments + 1
        comments(ncomments) = '|  Input file '//f_in(1:nc_f_in)
        ncomments = ncomments + 1
        comments(ncomments) = '|  digitized using program SMC_DIG,'
        ncomments = ncomments + 1
        comments(ncomments) = '|    with y_full_scale, nbits, dy = '
        write(comments(ncomments)(38:65),
     :      '(1p,e12.5,1x,0p,i2,1x,1p,e12.5)') 
     :       y_full_scale, nbits, dy
        ncomments = ncomments + 1
        comments(ncomments) = '|    and using '//type_of_a2d
        if (sigma_dither .eq. 0.0) then
          ncomments = ncomments + 1
          comments(ncomments) = 
     :     '|    with no dithering'
        else
          ncomments = ncomments + 1
          comments(ncomments) = 
     :     '|    with sigma_dither and iseed_dither = '
          write(comments(ncomments)(43:54),
     :      '(f6.3,1x,i5)') 
     :       sigma_dither, iseed_dither_in
        end if
        iflag = 0
        call datetime(date_c, time_c, iflag)
        ncomments = ncomments + 1
          comments(ncomments) = 
     :      '|   file '//f_out(1:nc_f_out)//
     :      ' written on '//date_c//' at time '//time_c
        ncomments = ncomments + 1
        comments(ncomments) = '|'
        int_head(16) = ncomments

         itype = -1  ! use info from header
        call SMCWrite(f_out(1:nc_f_out), precision,
     :             y_out, char_head, itype, 
     :                      int_head, real_head, comments)

        deallocate (y_in, y_out)

      go to 1000
      end

      include '\forprogs\get_lun.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smc_nsps.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\datetime.for'
      include '\forprogs\digitize.for'
      include '\forprogs\floor.for'
      include '\forprogs\gasdev.for'
      include '\forprogs\ran1.for'

 

            