! ------------------------------------------------------------------ smc_info
      Program smc_info

! Reads a list of smc or rs2 files, and for each it extracts station information
! and prints it to two files: 1) a summary file; and 2) a file in comma
! delimited format for import into data base programs.

! It is best to run this program within the folder containing the smc files;
! if path info is included, the f_smc character string may exceed the
! space allocated for it in the output

! Here is a sample control file:
!!Control file for program SMC_info:
!!Name of summary output file (suggest *.col):
! smc_info.col
!!Double space the col file (0=No; 1 = Yes)?:
! 0
!!Name of comma delimited output file (suggest *.txt, to import into Paradox):
! smc_info.txt
!!List of files:
!! a blank line indicates that two blank lines are inserted in the col output
!0433a_a.smc
!0433a_d.smc
!0437a_a.smc
!0437a_d.smc
!
!0598a_d.smc
!0598a_u.smc
!0598a_v.smc
!0598b_a.smc
!0598b_d.smc
!0598c_v.smc
!
!0601a_a.smc
!0601a_d.smc
!0601a_u.smc
!0601a_v.smc
!
!stop

!  Dates:  02/06/01 - Written by D. Boore, modified from SMC_STA
!          02/07/01 - Removed checks for new station, etc (see smc_sta 
!                     for that).  This also allowed me to remove the
!                     subroutines getinfo, output
!          03/20/01 - Add peak motion
!          05/23/01 - Add option to double space the col file output
!          05/24/01 - Insert 2 blank lines in output if find a blank line 
!                     in input.  This is useful for providing space 
!                     that can be used to fill in a station id, etc.
!          05/25/01 - Use skipcmnt to read comments.  Allow for 500 comments.
!          06/07/01 - Add type to output, and change column heading for warning
!                     from "ip" to "w"
!          06/07/01 - Combine with rs2_info so that one program reads either
!                     type.  I also abandoned smcread and rs2read, as there is
!                     no need to read the data themselves.  
!                     Also add more information to the comma delimited file,
!                     including the path of the directory from within which the
!                     program is run (no path is added if a path already
!                     exists, as determined by the existence of the 
!                     character "\"). 
!          06/08/01 - Search for a path.  Do not print the path in the *.col
!                     output.
!          07/23/01 - Changed itype to character*1 rather than integer in order 
!                     to deal with "*" in column 1 of line 1 (in Linda Seekins'
!                     DDS-7 files).
!          10/19/01 - Obtain default name of control file via CR
!          11/03/01 - Print r_ep rather than sta elev
!          04/11/02 - Check for unevenly-sampled data
!          05/21/02 - Added some write statements, inserted comments from 
!                     smc_sta.  Also, inserted check for adequate string 
!                     length in determining inst_type.  Files in 
!                     \psv\nr94_ms\scec were causing me lots of grief (probably
!                     because the format was not quite right, in particular the
!                     character headers.  I put in several checks for
!                     strings of inadequate length, but gave up at writing
!                     to nu_comma (I commented out the write statement).  I 
!                     should revisit operations with character strings when
!                     they have fewer than the required number of columns.
!          06/27/03 - Compiled using lf95, which means being more careful about
!                     file names (avoiding blanks at beginning and end)
!          12/10/03 - Add a decimal place to peak motion output
!          06/29/04 - Changed format of filter printout to not saturate for
!                     high-cut frequency.
!          07/16/04 - Allow for longer f_smc name
!          10/29/04 - squeeze in sps and npts
!          10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!          09/09/08 - Add az and vs30 to output
!          05/01/09 - Increase length of peak motion output
!          08/06/09 - Allow for longer file names in output.
!          04/27/11 - Add eq lat/long to output
!          02/02/12 - Compute r_ep if null and eq, sat coords not null
!          12/21/15 - Write peak motion using es11.4
!          11/05/16 - Allow for longer file name
!          03/16/17 - Increase length of current_dir


      character f_comma*300, f_col*300, quake_name*54, 
     :          sta_name*30, inst_type*10, data_source*45, itype*1
      character f_smc*300, f_out*300, path*300
      
      character buf_eq*19, buf_ds*6, buf_sn*30, current_dir*300
      character buf4*4

      character char_head(11)*80, comments(500)*80, cmnt(50)*80
      integer int_head(48)
      real real_head(50)

      real sta_lat, sta_long, eq_lat, eq_long
      real filter(4)
      integer id_sta, i_problem, i_dbl

      character f_ctl*40
      logical f_ctl_exist

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC_INFO.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) == '    ') f_ctl = 'smc_info.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnt, ncmnt)

!  Get the name of summary output file:

      f_col = ' '
      read(nu_ctl,'(a)') f_col
      call trim_c(f_col, nc_f_col)
      write(*,'(2a)') ' f_col = ', f_col(1:nc_f_col)
      call get_lun(nu_col)
      open (unit=nu_col, file=f_col(1:nc_f_col), status='unknown')

      write(nu_col,'(76x,"f_smc", 1x,"t", 2x,"eqid", 1x,"staid", 
     :      2x,"ins#", 
     :     19x,"station_name", 2x,"sta#",
     :     3x,"Vs30",
     :     4x,"stalat", 4x,"stalong", 
     :     1x,"source", 1x,"instr",
     :     5x,"earthquake", 
     :     5x,"eqlat", 5x,"eqlong", 
     :     1x,"r_ep",
     :     1x,"az:e2s",
     :     1x,"ivrt", 1x,"ihrz",
     :     1x,"fltr1", 2x,"fltr2", 1x,"flt3",
     :     1x,"flt4", 1x,"w",7x,"pkmtn",
     :     1x,"sps", 3x,"npts")') 

! Double space?

      call skipcmnt(nu_ctl, cmnt, ncmnt)
      read (nu_ctl,*) i_dbl

!  Get the name of comma-delimited output file:

      call skipcmnt(nu_ctl, cmnt, ncmnt)

      f_comma = ' '
      read(nu_ctl,'(a)') f_comma
      call trim_c(f_comma,nc_f_comma)
      write(*,'(2a)') ' f_comma = ', f_comma(1:nc_f_comma)
!      call get_lun(nu_comma)
!      open (unit=nu_comma, file=f_comma(1:nc_f_comma), 
!     :      status='unknown')

! Get current directory:
      call system('cd > crrntdir.lst')
      call get_lun(nu_cdir)
      open (unit=nu_cdir, file='crrntdir.lst', status='unknown')
      current_dir = ' '
      read(nu_cdir,'(a)') current_dir
      close(nu_cdir)
      call system('del crrntdir.lst')
      call trim_c(current_dir, nc_current_dir)
      current_dir(nc_current_dir+1:nc_current_dir+1) = '\'
      nc_current_dir = nc_current_dir + 1

! Read the control file, line by line:

      call skipcmnt(nu_ctl, cmnt, ncmnt) ! position to first entry

      loop over files: DO

        f_smc = ' '
        read (nu_ctl, '(a)') f_smc
        call trim_c(f_smc, nc_f_smc)
      
        if (f_smc(1:nc_f_smc) == ' ') then
          write(nu_col,*)
          write(nu_col,*)
          CYCLE loop over files
        end if

        buf4 = ' '
        buf4 = f_smc(1:4)
        call upstr(buf4)
        if (buf4(1:4) == 'STOP') then
          EXIT loop over files
        end if

      write(*,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)

      call ReadHdrs(f_smc, char_head, int_head, real_head, comments)

      quake_name = ' '
      quake_name = char_head(4)(27:80)
      sta_name = ' '
      sta_name = char_head(6)(11:40)
      inst_type = ' '
! CHeck for no info
      call trim_c(char_head(8), nc_ch8)
      if (nc_ch8 .ge. 20) then
        inst_type = char_head(8)(11:20)  ! see comments of 12/08/00 in smc_sta
      end if
!          12/08/00 - struggling with comparison
!                     of strings.  The problem was that in some smc files
!                     the inst_type was stored in columns beyond col. 15,
!                     which is a violation of the smc format.  I dealt with
!                     this by allowing the string to be in columns 11:20
!                     (and making the length of inst_type = 10)
!      inst_type = char_head(8)(11:15)
      if (inst_type == ' ') inst_type = 'blank'

! DEBUG
!      write(*,'(a)') ' After inst_type'
! DEBUG

      data_source = ' '
      if (nc_ch8 .ge. 36) then
        data_source = char_head(8)(36:nc_ch8)
      end if
      if (data_source == ' ') data_source = 'not specified'
      
! DEBUG
!      write(*,'(a)') ' After data_source'
! DEBUG

      itype = ' '
      itype = char_head(1)(1:1)

      inull     = int_head(1)
      i_rcrdr   = int_head(8)
      ivert     = int_head(13)
      ihoriz    = int_head(14)
      npts      = int_head(17)
      i_problem = int_head(18)
      id_sta    = int_head(30)
      ieqid     = int_head(40)
      istaid    = int_head(41)

      rnull     = real_head(1)
      sps       = real_head(2)
      eq_lat    = real_head(3)
      eq_long   = real_head(4)
      sta_lat   = real_head(11)
      sta_long  = real_head(12)
      sta_elev  = real_head(13)
      if (sta_elev == rnull) sta_elev = -999.0
      
      r_ep      = real_head(17)
      aze2s     = real_head(18)
      if (r_ep     == rnull) then
        if ( eq_lat /= rnull .and.  eq_long /= rnull .and.
     :      sta_lat /= rnull .and. sta_long /= rnull)     then
          call distaz( -1.0, eq_lat,eq_long,sta_lat,sta_long,
     :    rdeg, r_ep, aze2s, baz)
        else     
          r_ep     = -999.0
        end if
      end if
      
      if (aze2s     == rnull) aze2s     = -999.0
      
      ymax = real_head(30)
      ymin = real_head(32)
      pkmtn = amax1(abs(ymin), abs(ymax))

      do i = 1, 4
        filter(i) = real_head(i+32)
      end do

      vs30     = real_head(40)
      if (vs30     == rnull) vs30     = -999.0

      call trim_c(quake_name,nc_quake_name)
      call trim_c(sta_name,nc_sta_name)
      call trim_c(data_source,nc_data_source)
      call trim_c(inst_type,nc_inst_type)

! DEBUG
!      write(*,'(a)') ' After calls to trim_c'
! DEBUG

      buf_sn = ' '
      nc_sta_short = min0(30, nc_sta_name)
      if (nc_sta_short == 30) then
        buf_sn = sta_name(1:nc_sta_short)
      else
        buf_sn(1:30-nc_sta_short) = ' '
        buf_sn(30-nc_sta_short + 1:30) = sta_name(1:nc_sta_short)
      end if

      buf_ds = ' '
      nc_data_short = min0(6, nc_data_source)
      if (nc_data_short == 6) then
        buf_ds = data_source(1:nc_data_short)
      else
        buf_ds(1:6-nc_data_short) = ' '
        buf_ds(6-nc_data_short + 1:6) = data_source(1:nc_data_short)
      end if

      buf_eq = ' '
      nc_quake_short = min0(19, nc_quake_name)
      if (nc_quake_short == 19) then
        buf_eq = quake_name(1:nc_quake_short)
      else
        buf_eq(1:19-nc_quake_short) = ' '
        buf_eq(19-nc_quake_short + 1:19) = quake_name(1:nc_quake_short)
      end if

! DEBUG
!      write(*,'(a)') ' Before get_path'
! DEBUG

! Look for a path:
      call get_path_from_file_name(f_smc,nc_f_smc,path,nc_path)
 
! DEBUG
!      write(*,'(a)') ' After get_path'
! DEBUG

! Double space?
      if (i_dbl == 1) then
        write(nu_col,*)
      end if

      write(nu_col,'(1x,a80, 1x,a1,
     :               1x,i5,      1x,i5,    1x,i5,
     :               1x,a30,
     :               1x,i5,  
     :               1x,f6.1,
     :               1x,f9.5, 1x, f10.5,
     :               1x,a6,      1x,a5,
     :               1x,a14,     
     :               1x,f9.5, 1x, f10.5,
     :               1x,i4,
     :               1x,f6.1,
     :               1x,i4,      1x,i4,
     :               1x,f5.3, 1x,f6.3, 2(1x,f4.1), 1x,i1,
     :               1x,es11.4,
     :               1x,i3,1x,i6)') 
     : f_smc(nc_path+1:nc_f_smc), itype,
     : ieqid, istaid, i_rcrdr, 
     : buf_sn(1:30), 
     : id_sta, 
     : vs30, 
     : sta_lat, sta_long, 
     : buf_ds(1:6), inst_type(1:nc_inst_type),
     : buf_eq(1:19), 
     : eq_lat, eq_long, 
     : int(r_ep),
     : aze2s,
!     : buf_eq(1:19), int(sta_elev), 
     : ivert, ihoriz, 
     : (filter(i),i=1,4), i_problem,
     : pkmtn, int(sps), npts

! DEBUG
!      write(*,'(a)') ' After write_nu_col'
! DEBUG

      if (sps == rnull) then
        write(nu_col,'(a)') 
     :   ' *** For file above, sps = rnull; unevenly sampled! ***'
      end if

      if (sta_lat == rnull) sta_lat = 0.0
      if (sta_long == rnull) sta_long = 0.0

      f_out = ' '
      if (nc_path == 0) then
        f_out = current_dir(1:nc_current_dir)//f_smc(1:nc_f_smc)
      else
        f_out = f_smc(1:nc_f_smc)
      end if
      call trim_c(f_out,nc_f_out)     

!      write(nu_comma,'( 2(i6, ","), 
!     :                  """",a,""",",
!     :                  5x,a1,2(i6, ","), 
!     :                  """",a,""",", 2(i6, ","), 2(f10.5,","),
!     :                  i6, ",", 4("""",a,""","), 1p, e10.3)')
!     :    ieqid, istaid, 
!     :    f_out(1:nc_f_out), 
!     :    itype, ivert, ihoriz,
!     :    sta_name(1:nc_sta_name), id_sta, i_rcrdr, sta_lat, sta_long,
!     :    int(sta_elev), buf_ds(1:6), inst_type(1:nc_inst_type),
!     :    data_source(1:nc_data_source),
!     :    quake_name(1:nc_quake_name), pkmtn

! DEBUG
!      write(*,'(a)') ' After write_nu_comma'
! DEBUG

! Go back for a new file:
     
      END DO loop over files        

      close(nu_ctl)
      close(nu_col)
!      close(nu_comma)
      stop
      
      end
! ------------------------------------------------------------------ smc_info

! ------------------------------------------------------------------ ReadHdrs
      subroutine ReadHdrs(f_in, char_head, int_head, real_head, 
     :                    comments)

! Read headers of SMC and RS2 files

! The output arrays char_head, int_head, real_head should be dimensioned
! as follows in the calling program:

!      character char_head(11)*80, comments(500)*80
!      integer int_head(48)
!      real real_head(50)

!  Dates: 07/07/01 - Written by D. Boore, based on SMCRead

      real real_head(*)
      integer int_head(*)
      character*80 char_head(*), comments(*)
      character f_in*(*)

      
      call get_lun(nunit)
      open(unit= nunit, file=f_in, status='unknown')

      do i = 1, 11
        read(nunit, '(a)') char_head(i)
      end do

      read(nunit, '(8I10)') (int_head(i), i=1, 48)

      read(nunit, '(5e15.7)') (real_head(i), i = 1, 50)

      do i = 1, int_head(16)
        read(nunit,'(a)') comments(i)
      end do

      close(unit=nunit)

      return
      end
! ------------------------------------------------------------------ ReadHdrs


      include '\forprogs\skipcmnt.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\get_path_from_file_name.for'
      include '\forprogs\distaz.for'


   